import * as THREE from '../build/three.module.js';
import { GUI } from './jsm/libs/dat.gui.module.js';
import { OrbitControls } from './jsm/controls/OrbitControls.js';

class App {
    constructor() {
        this._initialize();
    }

    _initialize() {
        this.domWebGL = document.createElement('div');
        document.body.appendChild(this.domWebGL);

        let scene = new THREE.Scene();

        let renderer = new THREE.WebGLRenderer( { antialias: true });
        renderer.localClippingEnabled = true;
        this.domWebGL.appendChild(renderer.domElement);

        this.renderer = renderer;
        this.scene = scene;

        this._params = {
            clipIntersection: true,
            planeConstant: 0,
            showHelpers: false
        };

        this._setupCamera();
        this._setupModel();
        this._setupLights();
        this._setupControls();
        this._setupGUI();

        window.onresize = this.resize.bind(this);

        this.resize();
    }

    _setupGUI() {
        const params = this._params;

        const gui = new GUI();
        gui.add(params, 'clipIntersection').name('clip Intersection').onChange(
            (value) => {
                const children = this._group.children;
                for(let i=0; i<children.length; i++) {
                    children[i].material.clipIntersection = value;
                }
                this.render();
            }
        );

        gui.add(params, 'planeConstant', -1, 1).step(0.01).name('plane constant').onChange(
            (value) => {
                for(let j=0; j < this._clipPlanes.length; j++) {
                    this._clipPlanes[j].constant = value;
                }

                this.render();
            }
        );

        gui.add(params, 'showHelpers').name('show helpers').onChange(
            (value) => {
                this._helpers.visible = value;
                this.render();
            }
        );
    }

    _setupModel() {
        const clipPlanes = [
            new THREE.Plane(new THREE.Vector3(0, 1, 0), 0),
            new THREE.Plane(new THREE.Vector3(0, -1, 0), 0),
            new THREE.Plane(new THREE.Vector3(1, 0, 0), 0),
            new THREE.Plane(new THREE.Vector3(-1, 0, 0), 0),
            new THREE.Plane(new THREE.Vector3(0, 0, 1), 0),
        ];

        const group = new THREE.Group();
        
        for(let i=1; i<=30; i+=2) {
            const geometry = new THREE.SphereGeometry(i/30, 48, 24);
            const material = new THREE.MeshLambertMaterial({
                color: new THREE.Color().setHSL(Math.random(), 0.5, 0.5),
                side: THREE.DoubleSide,
                flatshade: true,
                clippingPlanes: clipPlanes,
                clipIntersection: this._params.clipIntersection
            });
            group.add(new THREE.Mesh(geometry, material));
        }

        this.scene.add(group);

        const helpers = new THREE.Group();
        for(let i=0; i<clipPlanes.length; i++) {
            helpers.add(new THREE.PlaneHelper(clipPlanes[i], 2, 0xffffff));
            helpers.visible = false;
            this.scene.add(helpers);
        }

        this._group = group;
        this._helpers = helpers;
        this._clipPlanes = clipPlanes;
    }

    _setupLights() {
        const light = new THREE.HemisphereLight(0xffffff, 0x080808, 1.5);
        light.position.set(-1.25, 1, 1.25);
        this.scene.add(light);
    }

    _setupCamera() {
        const camera = new THREE.PerspectiveCamera(40, window.innerWidth / window.innerHeight, 1, 200);
        camera.position.set(-1.5, 2.5, 3.0);

        this.scene.add(camera);

        this._camera = camera;
    }

    _setupControls() {
        const controls = new OrbitControls(this._camera, this.renderer.domElement);
        controls.addEventListener('change', this.render.bind(this));
        controls.minDistance = 1;
        controls.maxDistance = 10;
        controls.enablePan = false;
    }

    update(time) {

    }

    render(time) {
        //requestAnimationFrame(this.render.bind(this));

        this.update(time);
        this.renderer.render(this.scene, this._camera);
    }

    resize() {
        const aspect = window.innerWidth / window.innerHeight;

        this.renderer.setPixelRatio(window.devicePixelRatio);
        this.renderer.setSize(window.innerWidth, window.innerHeight);

        this._camera.aspect = aspect;
        this._camera.updateProjectionMatrix();

        this.render();
    }
}

window.onload = function () {
    (new App()).render();
}